﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Threading.Tasks;

namespace AsynchronousPrograming
{
    /// <summary>
    /// Practical usages of asynchronous operations 
    /// as desribed in Example01
    /// </summary>
    public static class Example02
    {
        private static readonly List<string> WebPagesToDownload = new List<string>
            {
                "http://www.google.com/",
                "http://msdn.microsoft.com/cs-cz/",
                "http://www.zive.cz/",
                "http://jizdnirady.idnes.cz/",
                "http://www.paypal.com",
                "http://www.seznam.cz/",
                "http://www.csfd.cz/",
                "http://www.youtube.com/",
                "http://www.idnes.cz/",
                "http://github.com",
                "http://graphsketch.com/",
                "http://uloz.to/",
                "http://en.wikipedia.org/wiki/Main_Page",
                "http://www.dailymotion.com/",
                "http://pctuning.tyden.cz/hardware",
                "http://auto.idnes.cz/",
                "http://mail.google.com",
                "http://www.netflix.com",
                "http://vimeo.com/",
                "http://www.hulu.com/",
                "http://www.twitch.tv/",
                "http://www.bbc.com/news",
                "http://edition.cnn.com/",
                "http://twitter.com/",
                "http://www.istockphoto.com/",
                "http://www.flickr.com/",
                "http://www.pexels.com/",
                "http://www.gettyimages.com/",
                "http://news.google.com/",
            };

        public static void Run()
        {
            // Practical example of async I/O operation
            GetNumberOfCharactersExample();
            Console.WriteLine("Performing some job in Run() method...");
            Console.ReadLine();

            // Practical example of multiple asynchronous network operations
            DownloadMultipleWebPagesAsync(WebPagesToDownload);
            Console.WriteLine("\nHEY, there's still stuff going on in the Run() method!\n");
            Console.ReadLine();
        }

        /// <summary>
        /// Perform GetNumberOfCharactersAsync() and prints its result
        /// </summary>
        private static async Task GetNumberOfCharactersExample()
        {
            var result = await GetNumberOfCharactersAsync();
            Console.WriteLine($"The result is: {result}" + Environment.NewLine +
                "Press any key to continue with the demonstration...");
        }

        /// <summary>
        /// Asynchronously retrieves number of characters in LoremIpsum.txt
        /// </summary>
        /// <returns></returns>
        private static async Task<int> GetNumberOfCharactersAsync()
        {
            string content;
            Console.WriteLine("GetNumberOfCharactersAsync has started...");
            using (var reader = new StreamReader(AppDomain.CurrentDomain.BaseDirectory + @"..\..\SampleData\LoremIpsum.txt"))
            {
                content = await reader.ReadToEndAsync();
            }
            Console.WriteLine("GetNumberOfCharactersAsync has finished...");
            return content.Length;
        }

        /// <summary>
        /// Asynchronously downloads web page from given URL
        /// </summary>
        /// <param name="url">url of the webpage to download</param>
        /// <returns>Web download promise</returns>
        private static async Task DownloadWebPageAsync(string url)
        {
            const int urlPrefixLength = 7;
            try
            {
                using (var webClient = new WebClient())
                {
                    Console.WriteLine($"Begin to download page: {url.Remove(0, urlPrefixLength)}");
                    var downloadToDirectory = AppDomain.CurrentDomain.BaseDirectory
                        + @"..\..\WebPages\" + url
                        .Remove(0, urlPrefixLength)
                        .Replace("/", "") + ".html";
                    webClient.Credentials = CredentialCache.DefaultNetworkCredentials;
                    await webClient.DownloadFileTaskAsync(new Uri(url), @downloadToDirectory);
                    Console.WriteLine($"Succesfully downloaded: {url.Remove(0, urlPrefixLength)}");
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine($"Download failed due to {ex.Message}");
            }
        }

        /// <summary>
        /// Downloads all websites according to given URLs
        /// </summary>
        /// <param name="urls">URLs of the webpages to download</param>
        private static async Task DownloadMultipleWebPagesAsync(IEnumerable<string> urls)
        {
            await Task.WhenAll(urls.Select(DownloadWebPageAsync));
        }
    }
}
